sem_core = function(params, delta_t, S_0, t, T_t) {
  
  # Extract the parameters from the vector params.  
  a = params[1]
  b = params[2]
  tau = params[3]
  
  # Identify the number of elements in the time vector t
  # (should match the number of elements in the temperature
  # vector T).  
  n_t = length(t)
  
  # Calculate the vector of equilibrium sea levels S_eq.  
  S_eq_t = a* T_t+ b
  
  # Step through time, calculating the sea level S for each
  # time step.  
  S_t = rep(S_0, length.out = n_t)
  for (i in 2: n_t) {
    S_t[i] = S_t[i- 1]+ (S_eq_t[i- 1]- S_t[i- 1])/ (tau/ delta_t)
  } # end for (i in 2: n_t)
  
  # Return the vector of sea levels S.  
  return(S_t)
  
} # end function sem_core

sem_rmse = function(params, delta_t, S_0, t, T_t, S_model_t) {
  
  # Call sem_core to generate the vector of estimated sea level
  # values, given the parameters.  
  S_t = sem_core(params, delta_t, S_0, t, T_t)
  
  # Calculate the root mean squared error between the "correct"
  # sea level values in S_model and the estimates in S.  
  rmse = sqrt(mean((S_t- S_model_t)^ 2))
  
  # Return the root mean squared error.  
  return(rmse)
  
}

sem_mad = function(params, delta_t, S_0, t, T_t, S_model_t) {
  
  # Call sem_core to generate the vector of estimated sea level
  # values, given the parameters.  
  S_t = sem_core(params, delta_t, S_0, t, T_t)
  
  # Calculate the root mean squared error between the "correct"
  # sea level values in S_model and the estimates in S.  
  mad = max(abs(S_t- S_model_t))
  
  # Return the root mean squared error.  
  return(mad)
  
}