# sem_test_v5.R
# Patrick Applegate, patrick.applegate@psu.edu
# 
# Generates Figure 4 from the manuscript.  

# Mise en place.  
rm(list = ls())
graphics.off()

# Set the working directory.  
# setwd('~/Documents/Work/geoengineering/sem_test')

# Load some packages.  
require('DEoptim')
require('RColorBrewer')

# Source some necessary functions.  
source('sem_functions.R')

# Read in the relevant data.  
t = read.csv('../data_tables/rcp8p5(2006_3000)_T.csv')[, 1]
T_rcp8p5_t = read.csv('../data_tables/rcp8p5(2006_3000)_T.csv')[, 2]
T_grad_t = read.csv('../data_tables/geo_0.1grad(2006_3000)_T.csv')[, 2: 11]
T_equil_t = read.csv('../data_tables/geo_equil(2006_3000)_T.csv')[, 2: 11]
S_rcp8p5_t = read.csv('../data_tables/rcp8p5(2006_3000)_V.csv')[, 2]
S_rcp8p5_t = S_rcp8p5_t[1]- S_rcp8p5_t
S_grad_t = read.csv('../data_tables/geo_0.1grad(2006_3000)_V.csv')[, 2: 11]
S_grad_t = S_grad_t[1, 1]- S_grad_t
S_equil_t = read.csv('../data_tables/geo_equil(2006_3000)_V.csv')[, 2: 11]
S_equil_t = S_equil_t[1, 1]- S_equil_t
delta_t = t[2]- t[1]

# Identify the indices where each of the geoengineering scenarios
# splits off from the base scenario.  
split_i = seq(20, 470, by = 50)

# Set up some colors.  
color = rev(brewer.pal(10, 'Spectral'))

# Fit the semi-empirical model to the results from SICOPOLIS.  
set.seed(1)
fit = DEoptim(sem_rmse, lower = c(10^-6, 10^-6, 10^-6), upper = c(10^1, 10^1, 10^5), control = DEoptim.control(itermax = 200, c = 0.1, trace = 10), delta_t, 0, t, T_rcp8p5_t, S_rcp8p5_t)
# fit = optim(c(1, 1, 200), sem_rmse, gr = NULL, delta_t, 0, t, T_rcp8p5_t, S_rcp8p5_t, lower = c(10^-6, 10^-6, 10^-6), upper = c(10^1, 10^1, 10^5))

# Calculate the best-fit curve from the semi-empirical model.  
S_best_rcp8p5_t = sem_core(fit$optim$bestmem, delta_t, 0, t, T_rcp8p5_t)
# S_best_rcp8p5_t = sem_core(fit$par, delta_t, 0, t, T_rcp8p5_t)

# Find the normalized mean average deviation between the curves from SICOPOLIS
# and the semi-empirical model at the points where the geoengineering scenarios
# split off.  
norm_mad = mean(abs(S_best_rcp8p5_t[split_i]- S_rcp8p5_t[split_i])/ S_rcp8p5_t[split_i])

# Calculate the results from the semi-empirical model using the best-fit 
# parameter values.  
S_best_grad_t = matrix(0, nrow = 995, ncol = 10)
S_best_equil_t = matrix(0, nrow = 995, ncol = 10)
for (i in 1: 10) {
  S_best_grad_t[, i] = sem_core(fit$optim$bestmem, delta_t, 0, t, T_grad_t[, i])
  # S_best_grad_t[, i] = sem_core(fit$par, delta_t, 0, t, T_grad_t[, i])
  S_best_equil_t[, i] = sem_core(fit$optim$bestmem, delta_t, 0, t, T_equil_t[, i])
}


# Make a figure.  
pdf('geoeng_fig4.pdf')
par(mfrow = c(2, 2))

matplot(t, S_equil_t, type = 'l', lty = 1, col = color, xlim = c(2000, 3000), ylim = c(0, max(c(S_rcp8p5_t, S_best_rcp8p5_t))), xlab = 'Time [yr]', ylab = 'Sea level contribution [m]', bty = 'n')
lines(t, S_rcp8p5_t, type = 'l', col = 'black')
points(t[split_i], S_rcp8p5_t[split_i], pch = 16, col = color)
matlines(t, S_best_equil_t, type = 'l', lty = 2, col = color)
lines(t, S_best_rcp8p5_t, type = 'l', lty = 2, col = 'black')
points(t[split_i], S_best_rcp8p5_t[split_i], col = color)

matplot(t, S_grad_t, type = 'l', lty = 1, col = color, xlim = c(2000, 3000), ylim = c(0, max(c(S_rcp8p5_t, S_best_rcp8p5_t))), xlab = 'Time [yr]', ylab = 'Sea level contribution [m]', bty = 'n')
lines(t, S_rcp8p5_t, type = 'l', col = 'black')
points(t[split_i], S_rcp8p5_t[split_i], pch = 16, col = color)
matlines(t, S_best_grad_t, type = 'l', lty = 2, col = color)
lines(t, S_best_rcp8p5_t, type = 'l', lty = 2, col = 'black')
points(t[split_i], S_best_rcp8p5_t[split_i], col = color)
dev.off()